module Event
  module Broker
    class << self
      def included(klass)
        klass.class_eval do
          extend  ClassMethods
          include InheritableAccessors
          include Handlers
        end
      end
    end
    
    module ClassMethods
      def handler_class_from_name(handler_name, base = self)
        name = "#{handler_name}_handler".camelize
        begin
          base.namespace.const_get(name)
        rescue NameError
          base.const_get(name)
        end
      end
    end

    module Handlers
      def self.included(klass)
        klass.class_eval do
          attr_reader :handlers
          inheritable_array_accessor :handler
        end
      end

      protected
        def initialize_handlers
          @handlers = self.class.handlers.map do |handler_name|
            self.class.handler_class_from_name(handler_name).new(self)
          end unless handlers
        end
    end
    
    def fire(event_name, memo = { })
      !catch(:stop_firing) do
        handlers.each do |handler|
          handler.handle(event_name, memo) if handler.handles?(event_name)
        end
      end.nil?
    end
  end
end
