module Event
  class Handler
    attr_reader :broker, :memo

    def initialize(broker)
      @broker = broker
    end

    def handles?(event_name)
      respond_to?(method_for_event_name(event_name))
    end

    def handle(event_name, memo)
      old_memo, @memo = @memo, memo
      send(method_for_event_name(event_name))
    ensure
      @memo = old_memo
    end

    def stop
      throw(:stop_firing)
    end

    def method_missing(method, *args, &block)
      broker.send(method, *args, &block)
    end

    protected
      def method_for_event_name(event_name)
        "on_#{event_name}"
      end
  end
end
