module InheritableAccessors
  def self.included(klass)
    klass.extend(ClassMethods)
  end
  
  module ClassMethods
    def inheritable_array_accessor(singular_collection_name)
      define_inheritable_accessor(singular_collection_name, [], :+, :concat)
    end
    
    def inheritable_hash_accessor(singular_collection_name)
      define_inheritable_accessor(singular_collection_name, { }, :merge, :merge!) { |args| args.first }
    end
    
    private
      def define_inheritable_accessor(singular_collection_name, default_value, nondestructive_merge_method, destructive_merge_method, &block)
        collection_name = "#{singular_collection_name}s"
        value_extractor = block ? block : Proc.new { |values| values }
        
        (class << self; self end).instance_eval do
          define_method(collection_name) do
            default_value
          end

          define_method(singular_collection_name) do |*values|
            (klass = class << self; self end).send(:define_method, collection_name) do
              (super rescue default_value).send(nondestructive_merge_method, retrieve_inheritable_object(klass, collection_name, default_value))
            end
            retrieve_inheritable_object(klass, collection_name, default_value).send(destructive_merge_method, value_extractor.call(values))
          end
        end
      end 
      
      def retrieve_inheritable_object(klass, collection_name, default_value)
        instance_variable = "@#{collection_name}"
        klass.instance_variable_get(instance_variable) || klass.instance_variable_set(instance_variable, default_value.dup)
      end
  end
end
