require "socket"

module IRC
  class Connection
    attr_reader :server, :port, :options
    
    def initialize(server, port, options = { })
      @server, @port, @options = server, port, options
      @socket = TCPSocket.open(server, port)
    end
    
    def connected?
      !@socket.eof?
    end
    
    def each_response
      @socket.each_line do |line|
        response = Response.parse(line)
        p response
        yield response
      end
    end
    
    def disconnect
      @socket.close
    end
    
    def send_line(line)
      puts ">>> #{line}"
      @socket.puts(line)
    end
    
    def request(command, *args)
      text = args.last.is_a?(Hash) ? ":#{args.pop[:text]}" : nil
      args = args.any? ? args.map { |arg| arg.to_s.strip }.join(" ") : nil
      line = [command.to_s.upcase, args, text].compact.join(" ")
      send_line(line)
    end
  end
end
